#include "hamming_16_11.h"

/*
generator matrix accodring to etsi B.3.4

1 0 0 0 0 0 0 0 0 0 0 1 0 0 1 1
0 1 0 0 0 0 0 0 0 0 0 1 1 0 1 0
0 0 1 0 0 0 0 0 0 0 0 1 1 1 1 1
0 0 0 1 0 0 0 0 0 0 0 1 1 1 0 0
0 0 0 0 1 0 0 0 0 0 0 0 1 1 1 0
0 0 0 0 0 1 0 0 0 0 0 1 0 1 0 1
0 0 0 0 0 0 1 0 0 0 0 0 1 0 1 1
0 0 0 0 0 0 0 1 0 0 0 1 0 1 1 0
0 0 0 0 0 0 0 0 1 0 0 1 1 0 0 1
0 0 0 0 0 0 0 0 0 1 0 0 1 1 0 1
0 0 0 0 0 0 0 0 0 0 1 0 0 1 1 1

parity check matrix according to H = [-PT|In-k]

1 1 1 1 0 1 0 1 1 0 0 1 0 0 0 0
0 1 1 1 1 0 1 0 1 1 0 0 1 0 0 0
0 0 1 1 1 1 0 1 0 1 1 0 0 1 0 0
1 1 1 0 1 0 1 1 0 0 1 0 0 0 1 0
1 0 1 0 0 1 1 0 1 1 1 0 0 0 0 1
*/

// parity check matrix as code
uint16_t hamming_16_11_parity_check_matrix[] = {
    0b1111010110010000,
    0b0111101011001000,
    0b0011110101100100,
    0b1110101100100010,
    0b1010011011100001
};

struct correction {
    uint16_t syndrome;
    uint16_t error_pattern;
};

// generated by applying possible combinations of errors
static struct correction corrections[] = {
    { 1, 1 },
    { 2, 2 },
    { 4, 4 },
    { 8, 8 },
    { 16, 16 },
    { 7, 32 },
    { 13, 64 },
    { 25, 128 },
    { 22, 256 },
    { 11, 512 },
    { 21, 1024 },
    { 14, 2048 },
    { 28, 4096 },
    { 31, 8192 },
    { 26, 16384 },
    { 19, 32768 }
};

uint16_t hamming_16_11_parity(uint16_t* data) {
    uint16_t parity = 0;

    uint8_t k;
    for (k = 0; k < 5; k++) {
        uint8_t bit = 0, l;
        for (l = 0; l < 16; l++) {
            if ((hamming_16_11_parity_check_matrix[k] >> l) & 1) {
                bit ^= ((*data >> l) & 1);
            }
        }

        parity = (parity << 1) | (bit & 1);
    }

    return parity;
}

bool hamming_16_11(uint16_t* data) {
    uint16_t parity = hamming_16_11_parity(data);

    if (parity == 0) return true;

    int num_corrections = sizeof(corrections)/sizeof(corrections[0]);
    for (int i = 0; i < num_corrections; i++) {
        if (corrections[i].syndrome == parity) {
            *data = *data ^ corrections[i].error_pattern;
            return true;
        }
    }

    return false;
}
