#include "hamming_7_4.h"

/*
generator matrix accordint to etsi B.3.5
1 0 0 0 1 0 1
0 1 0 0 1 1 1
0 0 1 0 1 1 0
0 0 0 1 0 1 1

parity check matrix according to H = [-PT|In-k]

1 1 1 0 1 0 0
0 1 1 1 0 1 0
1 1 0 1 0 0 1
*/

// parity check matrix as code
uint8_t hamming_7_4_parity_check_matrix[] = {
    0b01110100,
    0b00111010,
    0b01101001
};

struct correction {
    uint16_t syndrome;
    uint16_t error_pattern;
};

// generated by applying possible combinations of errors
static struct correction corrections[] = {
    { 1, 1 },
    { 2, 2 },
    { 4, 4 },
    { 3, 8 },
    { 6, 16 },
    { 7, 32 },
    { 5, 64 }
};

uint8_t hamming_7_4_parity(uint8_t* data) {
    uint8_t parity = 0;

    uint8_t k;
    for (k = 0; k < 3; k++) {
        uint8_t bit = 0, l;
        for (l = 0; l < 7; l++) {
            if ((hamming_7_4_parity_check_matrix[k] >> l) & 1) {
                bit ^= ((*data >> l) & 1);
            }
        }

        parity = (parity << 1) | (bit & 1);
    }

    return parity;
}

bool hamming_7_4(uint8_t* data) {
    uint8_t parity = hamming_7_4_parity(data);

    if (parity == 0) return true;

    int num_corrections = sizeof(corrections)/sizeof(corrections[0]);
    for (int i = 0; i < num_corrections; i++) {
        if (corrections[i].syndrome == parity) {
            *data = *data ^ corrections[i].error_pattern;
            return true;
        }
    }

    return false;
}
